@@ -114,6 +114,15 @@ extern void tracker_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                              gpointer      marshal_data);
 #define tracker_marshal_BOOL__OBJECT	tracker_marshal_BOOLEAN__OBJECT
 
+/* BOOL:OBJECT,BOXED,BOXED,OBJECT (tracker-marshal.list:14) */
+extern void tracker_marshal_BOOLEAN__OBJECT_BOXED_BOXED_OBJECT (GClosure     *closure,
+                                                                GValue       *return_value,
+                                                                guint         n_param_values,
+                                                                const GValue *param_values,
+                                                                gpointer      invocation_hint,
+                                                                gpointer      marshal_data);
+#define tracker_marshal_BOOL__OBJECT_BOXED_BOXED_OBJECT	tracker_marshal_BOOLEAN__OBJECT_BOXED_BOXED_OBJECT
+
 G_END_DECLS
 
 #endif /* __tracker_marshal_MARSHAL_H__ */
