@@ -482,6 +482,40 @@ id3v2tag_free (id3v2tag *tags)
 	g_free (tags->title3);
 }
 
+static gboolean
+guess_dlna_profile (gint          bitrate,
+                    gint          frequency,
+                    gint          mpeg_version,
+                    gint          layer_version,
+                    gint          n_channels,
+                    const gchar **dlna_profile,
+                    const gchar **dlna_mimetype)
+{
+	if (mpeg_version == MPEG_V1 &&
+	    layer_version == LAYER_3 &&
+	    (bitrate >= 32000 && bitrate <= 320000) &&
+	    (n_channels == 1 || n_channels == 2) &&
+	    (frequency == freq_table[0][0] ||
+	     frequency == freq_table[1][0] ||
+	     frequency == freq_table[2][0])) {
+		*dlna_profile = "MP3";
+		*dlna_mimetype = "audio/mpeg";
+		return TRUE;
+	}
+
+	if ((bitrate >= 8000 && bitrate <= 320000) &&
+	    (mpeg_version == MPEG_V1 || mpeg_version == MPEG_V2) &&
+	    (frequency == freq_table[0][0] || frequency == freq_table[0][1] ||
+	     frequency == freq_table[1][0] || frequency == freq_table[1][1] ||
+	     frequency == freq_table[2][0] || frequency == freq_table[2][1])) {
+		*dlna_profile = "MP3X";
+		*dlna_mimetype = "audio/mpeg";
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static char *
 read_id3v1_buffer (int     fd,
                    goffset size)
@@ -817,6 +851,7 @@ mp3_parse_header (const gchar          *data,
                   TrackerSparqlBuilder *metadata,
                   MP3Data              *filedata)
 {
+	const gchar *dlna_profile, *dlna_mimetype;
 	guint header;
 	gchar mpeg_ver = 0;
 	gchar layer_ver = 0;
@@ -831,6 +866,7 @@ mp3_parse_header (const gchar          *data,
 	guint frame_size;
 	guint frames = 0;
 	size_t pos = 0;
+	gint n_channels;
 
 	pos = seek_pos;
 
@@ -932,12 +968,10 @@ mp3_parse_header (const gchar          *data,
 	tracker_sparql_builder_predicate (metadata, "nfo:codec");
 	tracker_sparql_builder_object_string (metadata, "MPEG");
 
+	n_channels = ((header & ch_mask) == ch_mask) ? 1 : 2;
+
 	tracker_sparql_builder_predicate (metadata, "nfo:channels");
-	if ((header & ch_mask) == ch_mask) {
-		tracker_sparql_builder_object_int64 (metadata, 1);
-	} else {
-		tracker_sparql_builder_object_int64 (metadata, 2);
-	}
+	tracker_sparql_builder_object_int64 (metadata, n_channels);
 
 	avg_bps /= frames;
 
@@ -958,6 +992,13 @@ mp3_parse_header (const gchar          *data,
 	tracker_sparql_builder_predicate (metadata, "nfo:averageBitrate");
 	tracker_sparql_builder_object_int64 (metadata, avg_bps*1000);
 
+	if (guess_dlna_profile (bitrate, sample_rate,
+	                        mpeg_ver, layer_ver, n_channels,
+	                        &dlna_profile, &dlna_mimetype)) {
+		tracker_sparql_builder_predicate (metadata, "nmm:dlnaProfile");
+		tracker_sparql_builder_object_string (metadata, dlna_profile);
+	}
+
 	return TRUE;
 }
 
@@ -1016,8 +1057,16 @@ id3v2_strlen (const gchar  encoding,
 	switch (encoding) {
 	case 0x01:
 	case 0x02:
+		
 		/* UTF-16, string terminated by two NUL bytes */
-		pos = memmem (text, len, "\0\0", 2);
+		pos = memmem (text, len, "\0\0\0", 3);
+
+		if (pos == NULL) {
+			pos = memmem (text, len, "\0\0", 2);
+		} else {
+			pos++;
+		}
+
 		if (pos != NULL) {
 			return pos - text;
 		} else {
@@ -1031,7 +1080,8 @@ id3v2_strlen (const gchar  encoding,
 static gchar *
 id3v24_text_to_utf8 (const gchar  encoding,
                      const gchar *text,
-                     gssize       len)
+                     gssize       len,
+                     id3tag      *info)
 {
 	/* This byte describes the encoding
 	 * try to convert strings to UTF-8
@@ -1047,7 +1097,7 @@ id3v24_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	case 0x01 :
 		return convert_to_encoding (text,
@@ -1072,7 +1122,7 @@ id3v24_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	}
 }
@@ -1080,7 +1130,8 @@ id3v24_text_to_utf8 (const gchar  encoding,
 static gchar *
 id3v2_text_to_utf8 (const gchar  encoding,
                     const gchar *text,
-                    gssize       len)
+                    gssize       len,
+                    id3tag      *info)
 {
 	/* This byte describes the encoding
 	 * try to convert strings to UTF-8
@@ -1096,7 +1147,7 @@ id3v2_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	case 0x01 :
 		/* return g_convert (text, */
@@ -1114,7 +1165,7 @@ id3v2_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	}
 }
@@ -1232,7 +1283,7 @@ get_id3v24_tags (id3v24frame           frame,
 		offset        = 4 + text_desc_len + id3v2_nul_size (text_encode);
 		text          = &data[pos + offset]; /* <full text string according to encoding> */
 
-		word = id3v24_text_to_utf8 (text_encode, text, csize - offset);
+		word = id3v24_text_to_utf8 (text_encode, text, csize - offset, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1248,7 +1299,7 @@ get_id3v24_tags (id3v24frame           frame,
 		gchar *word;
 
 		/* text frames */
-		word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
 		}
@@ -1418,7 +1469,7 @@ get_id3v23_tags (id3v24frame           frame,
 		offset        = 4 + text_desc_len + id3v2_nul_size (text_encode);
 		text          = &data[pos + offset]; /* <full text string according to encoding> */
 
-		word = id3v2_text_to_utf8 (text_encode, text, csize - offset);
+		word = id3v2_text_to_utf8 (text_encode, text, csize - offset, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1435,7 +1486,7 @@ get_id3v23_tags (id3v24frame           frame,
 		gchar *word;
 
 		/* text frames */
-		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1581,7 +1632,7 @@ get_id3v20_tags (id3v2frame            frame,
 		/* text frames */
 		gchar *word;
 
-		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
 		}
@@ -2397,10 +2448,6 @@ extract_mp3 (const gchar          *uri,
 
 	g_free (md.album_uri);
 
-	/* FIXME We use a hardcoded value here for now. In reality there's a second option MP3X */
-	tracker_sparql_builder_predicate (metadata, "nmm:dlnaProfile");
-	tracker_sparql_builder_object_string (metadata, "MP3");
-
 	/* Get mp3 stream info */
 	mp3_parse (buffer, buffer_size, audio_offset, uri, metadata, &md);
 
