@@ -28,8 +28,7 @@
 
 #include <libtracker-common/tracker-log.h>
 
-#include "tracker-writeback-dispatcher.h"
-#include "tracker-writeback-consumer.h"
+#include "tracker-writeback.h"
 #include "tracker-config.h"
 
 #define ABOUT	  \
@@ -42,8 +41,11 @@
 	"\n" \
 	"  http://www.gnu.org/licenses/gpl.txt\n"
 
+#define QUIT_TIMEOUT 30 /* 1/2 minutes worth of seconds */
+
 static gboolean      version;
 static gint          verbosity = -1;
+static gboolean      disable_shutdown;
 
 static GOptionEntry  entries[] = {
 	{ "version", 'V', 0,
@@ -55,96 +57,15 @@ static GOptionEntry  entries[] = {
 	  N_("Logging, 0 = errors only, "
 	     "1 = minimal, 2 = detailed and 3 = debug (default=0)"),
 	  NULL },
+	/* Debug run is used to avoid that the mainloop exits, so that
+	 * as a developer you can be relax when running the tool in gdb */
+	{ "disable-shutdown", 'd', 0,
+	  G_OPTION_ARG_NONE, &disable_shutdown,
+	  N_("Disable shutting down after 30 seconds of inactivity"),
+	  NULL },
 	{ NULL }
 };
 
-typedef struct {
-	gint    subject;
-	GArray *rdf_types;
-} WritebackData;
-
-static TrackerWritebackConsumer *consumer = NULL;
-static TrackerWritebackDispatcher *dispatcher = NULL;
-static GMainContext *dispatcher_context = NULL;
-
-static WritebackData *
-writeback_data_new (gint    subject,
-                    GArray *rdf_types)
-{
-	WritebackData *data;
-	guint i;
-
-	data = g_slice_new (WritebackData);
-	data->subject = subject;
-
-	data->rdf_types = g_array_sized_new (FALSE, FALSE, sizeof (gint), rdf_types->len);
-
-	for (i = 0; i < rdf_types->len; i++) {
-		gint id = g_array_index (rdf_types, gint, i);
-		g_array_append_val (data->rdf_types, id);
-	}
-
-	return data;
-}
-
-static void
-writeback_data_free (WritebackData *data)
-{
-	g_array_free (data->rdf_types, TRUE);
-	g_slice_free (WritebackData, data);
-}
-
-/* This function will be executed in the main thread */
-static gboolean
-on_writeback_idle_cb (gpointer user_data)
-{
-	WritebackData *data = user_data;
-
-	g_message ("Main thread (%p) got signaled of writeback petition", g_thread_self ());
-
-	tracker_writeback_consumer_add_subject (consumer, data->subject, data->rdf_types);
-
-	writeback_data_free (data);
-
-	return FALSE;
-}
-
-/* This callback run in the dispatcher thread */
-static void
-on_writeback_cb (TrackerWritebackDispatcher *dispatcher,
-                 gint                        subject,
-                 GArray                     *rdf_types)
-{
-	WritebackData *data;
-
-	g_message ("Got writeback petition on thread '%p' for subject '%d'",
-	           g_thread_self (), subject);
-
-	data = writeback_data_new (subject, rdf_types);
-	g_idle_add_full (G_PRIORITY_HIGH_IDLE,
-	                 on_writeback_idle_cb,
-	                 data, NULL);
-}
-
-static gpointer
-dispatcher_thread_func (gpointer data)
-{
-	GMainLoop *loop;
-
-	g_message ("DBus Dispatcher thread created: %p", g_thread_self ());
-
-	g_signal_connect (dispatcher, "writeback",
-	                  G_CALLBACK (on_writeback_cb), NULL);
-
-	loop = g_main_loop_new (dispatcher_context, FALSE);
-	g_main_loop_run (loop);
-
-	g_object_unref (dispatcher);
-	g_main_loop_unref (loop);
-	g_main_context_unref (dispatcher_context);
-
-	return NULL;
-}
 
 static void
 sanity_check_option_values (TrackerConfig *config)
@@ -159,13 +80,14 @@ main (int   argc,
       char *argv[])
 {
 	TrackerConfig *config;
+	TrackerController *controller;
 	GOptionContext *context;
 	GMainLoop *loop;
 	GError *error = NULL;
 	gchar *log_filename;
+	guint shutdown_timeout;
 
 	g_thread_init (NULL);
-	dbus_g_thread_init ();
 
 	g_type_init ();
 
@@ -204,38 +126,17 @@ main (int   argc,
 
 	sanity_check_option_values (config);
 
-
-	consumer = tracker_writeback_consumer_new (&error);
-
-	if (error) {
-		g_critical ("Error creating consumer: %s", error->message);
-		g_error_free (error);
-
-		return EXIT_FAILURE;
+	if (disable_shutdown) {
+		shutdown_timeout = 0;
+	} else {
+		shutdown_timeout = QUIT_TIMEOUT;
 	}
 
-	/* Create dispatcher thread data here, GType
-	 * initialization for boxed types don't seem
-	 * to be threadsafe, this is troublesome with
-	 * signals initialization. */
-
-	dispatcher_context = g_main_context_new ();
-	dispatcher = tracker_writeback_dispatcher_new (dispatcher_context,
-	                                               &error);
+	controller = tracker_controller_new (shutdown_timeout, &error);
 
 	if (error) {
-		g_critical ("Error creating dispatcher: %s", error->message);
+		g_critical ("Error creating controller: %s", error->message);
 		g_error_free (error);
-
-		return EXIT_FAILURE;
-	}
-
-	g_thread_create (dispatcher_thread_func, dispatcher, FALSE, &error);
-
-	if (error) {
-		g_critical ("Error creating dispatcher thread: %s", error->message);
-		g_error_free (error);
-
 		return EXIT_FAILURE;
 	}
 
@@ -244,9 +145,11 @@ main (int   argc,
 	loop = g_main_loop_new (NULL, FALSE);
 	g_main_loop_run (loop);
 
+
 	tracker_log_shutdown ();
 
-	g_object_unref (consumer);
+	g_object_unref (controller);
+
 	g_main_loop_unref (loop);
 
 	g_object_unref (config);
